@@ -2,9 +2,9 @@
  * @file inst.c
  * SQLite ODBC Driver installer/uninstaller for WIN32
  *
- * $Id: inst.c,v 1.7 2006/04/03 15:22:25 chw Exp chw $
+ * $Id: inst.c,v 1.11 2008/12/06 10:53:19 chw Exp chw $
  *
- * Copyright (c) 2001-2006 Christian Werner <chw@ch-werner.de>
+ * Copyright (c) 2001-2007 Christian Werner <chw@ch-werner.de>
  *
  * See the file "license.terms" for information on usage
  * and redistribution of this file and for a
@@ -35,8 +35,16 @@ static char *DriverDLL[3] = {
     "sqliteodbcu.dll",
     "sqlite3odbc.dll"
 };
+#ifdef WITH_SQLITE_DLLS
+static char *EngineDLL[3] = {
+    "sqlite.dll",
+    "sqliteu.dll",
+    "sqlite3.dll"
+};
+#endif
 
 static int quiet = 0;
+static int nosys = 0;
 
 /**
  * Handler for ODBC installation error messages.
@@ -66,18 +74,68 @@ ProcessErrorMessages(char *name)
     return ret;
 }
 
+/**
+ * Copy or delete SQLite3 module DLLs
+ * @param dllname file name of driver DLL
+ * @param path install directory for modules
+ * @param del flag, when true, delete DLLs in install directory
+ */
+
+static BOOL
+CopyOrDelModules(char *dllname, char *path, BOOL del)
+{
+    char firstpat[MAX_PATH];
+    WIN32_FIND_DATA fdata;
+    HANDLE h;
+    DWORD err;
+
+    if (strncmp(dllname, "sqlite3", 7)) {
+	return TRUE;
+    }
+    firstpat[0] = '\0';
+    if (del) {
+	strcpy(firstpat, path);
+	strcat(firstpat, "\\");
+    }
+    strcat(firstpat, "sqlite3_mod*.dll");
+    h = FindFirstFile(firstpat, &fdata);
+    if (h == INVALID_HANDLE_VALUE) {
+	return TRUE;
+    }
+    do {
+	if (del) {
+	    DeleteFile(fdata.cFileName);
+	} else {
+	    char buf[1024];
+
+	    sprintf(buf, "%s\\%s", path, fdata.cFileName);
+	    if (!CopyFile(fdata.cFileName, buf, 0)) {
+		sprintf(buf, "Copy %s to %s failed", fdata.cFileName, path);
+		MessageBox(NULL, buf, "CopyFile",
+			   MB_ICONSTOP|MB_OK|MB_TASKMODAL|MB_SETFOREGROUND); 
+		FindClose(h);
+		return FALSE;
+	    }
+	}
+    } while (FindNextFile(h, &fdata));
+    err = GetLastError();
+    FindClose(h);
+    return err == ERROR_NO_MORE_FILES;
+}
+
 /**
  * Driver installer/uninstaller.
  * @param remove true for uninstall
  * @param drivername print name of driver
  * @param dllname file name of driver DLL
+ * @param dll2name file name of additional DLL
  * @param dsname name for data source
  */
 
 static BOOL
-InUn(int remove, char *drivername, char *dllname, char *dsname)
+InUn(int remove, char *drivername, char *dllname, char *dll2name, char *dsname)
 {
-    char path[301], driver[300], attr[300], inst[400];
+    char path[301], driver[300], attr[300], inst[400], inst2[400];
     WORD pathmax = sizeof (path) - 1, pathlen;
     DWORD usecnt, mincnt;
 
@@ -90,7 +148,6 @@ InUn(int remove, char *drivername, char *dllname, char *dsname)
 	while (*p) {
 	    if (*p == ';') {
 		*p = '\0';
-		++p;
 	    }
 	    ++p;
 	}
@@ -103,15 +160,21 @@ InUn(int remove, char *drivername, char *dllname, char *dsname)
 	while (*p) {
 	    if (*p == ';') {
 		*p = '\0';
-		++p;
 	    }
 	    ++p;
 	}
 	sprintf(inst, "%s\\%s", path, dllname);
+	if (dll2name) {
+	    sprintf(inst2, "%s\\%s", path, dll2name);
+	}
 	if (!remove && usecnt > 0) {
 	    /* first install try: copy over driver dll, keeping DSNs */
 	    if (GetFileAttributes(dllname) != 0xFFFFFFFF &&
-		CopyFile(dllname, inst, 0)) {
+		CopyFile(dllname, inst, 0) &&
+		CopyOrDelModules(dllname, path, 0)) {
+		if (dll2name != NULL) {
+		    CopyFile(dll2name, inst2, 0);
+		}
 		return TRUE;
 	    }
 	}
@@ -130,6 +193,8 @@ InUn(int remove, char *drivername, char *dllname, char *dsname)
 		char buf[512];
 
 		DeleteFile(inst);
+		/* but keep inst2 */
+		CopyOrDelModules(dllname, path, 1);
 		if (!quiet) {
 		    sprintf(buf, "%s uninstalled.", drivername);
 		    MessageBox(NULL, buf, "Info",
@@ -137,17 +202,19 @@ InUn(int remove, char *drivername, char *dllname, char *dsname)
 			       MB_SETFOREGROUND);
 		}
 	    }
+	    if (nosys) {
+		goto done;
+	    }
 	    sprintf(attr, "DSN=%s;Database=sqlite.db;", dsname);
 	    p = attr;
 	    while (*p) {
 		if (*p == ';') {
 		    *p = '\0';
-		    ++p;
 		}
 		++p;
 	    }
 	    SQLConfigDataSource(NULL, ODBC_REMOVE_SYS_DSN, drivername, attr);
-	    return TRUE;
+	    goto done;
 	}
 	if (GetFileAttributes(dllname) == 0xFFFFFFFF) {
 	    return FALSE;
@@ -160,17 +227,30 @@ InUn(int remove, char *drivername, char *dllname, char *dsname)
 		       MB_ICONSTOP|MB_OK|MB_TASKMODAL|MB_SETFOREGROUND); 
 	    return FALSE;
 	}
+	if (dll2name != NULL && !CopyFile(dll2name, inst2, 0)) {
+	    char buf[512];
+
+	    sprintf(buf, "Copy %s to %s failed", dll2name, inst2);
+	    MessageBox(NULL, buf, "CopyFile",
+		       MB_ICONSTOP|MB_OK|MB_TASKMODAL|MB_SETFOREGROUND); 
+	    /* but go on hoping that an SQLite engine is in place */
+	}
+	if (!CopyOrDelModules(dllname, path, 0)) {
+	    return FALSE;
+	}
 	if (!SQLInstallDriverEx(driver, path, path, pathmax, &pathlen,
 				ODBC_INSTALL_COMPLETE, &usecnt)) {
 	    ProcessErrorMessages("SQLInstallDriverEx");
 	    return FALSE;
 	}
+	if (nosys) {
+	    goto done;
+	}
 	sprintf(attr, "DSN=%s;Database=sqlite.db;", dsname);
 	p = attr;
 	while (*p) {
 	    if (*p == ';') {
 		*p = '\0';
-		++p;
 	    }
 	    ++p;
 	}
@@ -183,6 +263,7 @@ InUn(int remove, char *drivername, char *dllname, char *dsname)
 	ProcessErrorMessages("SQLInstallDriverManager");
 	return FALSE;
     }
+done:
     return TRUE;
 }
 
@@ -217,8 +298,14 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
     }
     remove = strstr(p, "uninst") != NULL;
     quiet = strstr(p, "instq") != NULL;
+    nosys = strstr(p, "nosys") != NULL;
     for (i = 0; i < 3; i++) {
-	ret[i] = InUn(remove, DriverName[i], DriverDLL[i], DSName[i]);
+#ifdef WITH_SQLITE_DLLS
+	p = EngineDLL[i];
+#else
+	p = NULL;
+#endif
+	ret[i] = InUn(remove, DriverName[i], DriverDLL[i], p, DSName[i]);
     }
     if (!remove && (ret[0] || ret[1] || ret[2])) {
 	if (!quiet) {
