@@ -15,16 +15,16 @@
  * @file sqlite3odbc.h
  * Header file for SQLite3 ODBC driver.
  *
- * $Id: sqlite3odbc.h,v 1.16 2006/07/23 08:10:03 chw Exp chw $
+ * $Id: sqlite3odbc.h,v 1.27 2009/01/08 11:57:14 chw Exp chw $
  *
- * Copyright (c) 2004-2006 Christian Werner <chw@ch-werner.de>
+ * Copyright (c) 2004-2009 Christian Werner <chw@ch-werner.de>
  *
  * See the file "license.terms" for information on usage
  * and redistribution of this file and for a
  * DISCLAIMER OF ALL WARRANTIES.
  */
 
-#ifdef _WIN32
+#if defined(_WIN32) || defined(_WIN64)
 #include <windows.h>
 #include <stdio.h>
 #include <io.h>
@@ -32,13 +32,14 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <stdio.h>
-#include <stdlib.h>
 #include <unistd.h>
 #endif
-#if defined(HAVE_LOCALECONV) || defined(_WIN32)
+#include <stdlib.h>
+#if defined(HAVE_LOCALECONV) || defined(_WIN32) || defined(_WIN64)
 #include <locale.h>
 #endif
 #include <stdarg.h>
+#include <stddef.h>
 #include <string.h>
 #include <sql.h>
 #include <sqlext.h>
@@ -48,7 +49,7 @@
 #ifdef HAVE_IODBC
 #include <iodbcinst.h>
 #endif
-#if defined(HAVE_UNIXODBC) || defined(_WIN32)
+#if defined(HAVE_UNIXODBC) || defined(_WIN32) || defined(_WIN64)
 #include <odbcinst.h>
 #endif
 
@@ -70,6 +71,18 @@
 #define SQLROWCOUNT SQLUINTEGER
 #endif
 
+#ifndef HAVE_SQLSETPOSIROW
+#define SQLSETPOSIROW SQLUSMALLINT
+#endif
+
+#ifndef HAVE_SQLROWOFFSET
+#define SQLROWOFFSET SQLLEN
+#endif
+
+#ifndef HAVE_SQLROWSETSIZE
+#define SQLROWSETSIZE SQLULEN
+#endif
+
 struct dbc;
 struct stmt;
 
@@ -82,6 +95,10 @@ struct stmt;
 typedef struct {
     int magic;			/**< Magic cookie */
     int ov3;			/**< True for SQL_OV_ODBC3 */
+#if defined(_WIN32) || defined(_WIN64)
+    CRITICAL_SECTION cs;	/**< For serializing most APIs */
+    DWORD owner;		/**< Current owner of CS or 0 */
+#endif
     struct dbc *dbcs;		/**< Pointer to first DBC */
 } ENV;
 
@@ -101,6 +118,7 @@ typedef struct dbc {
     char *dsn;			/**< ODBC data source name */
     int timeout;		/**< Lock timeout value */
     long t0;			/**< Start time for SQLITE busy handler */
+    int busyint;		/**< Interrupt busy handler from SQLCancel() */
     int *ov3;			/**< True for SQL_OV_ODBC3 */
     int ov3val;			/**< True for SQL_OV_ODBC3 */
     int autocommit;		/**< Auto commit state */
@@ -109,6 +127,8 @@ typedef struct dbc {
     int naterr;			/**< Native error code */
     char sqlstate[6];		/**< SQL state for SQLError() */
     SQLCHAR logmsg[1024];	/**< Message for SQLError() */
+    int nowchar;		/**< Don't try to use WCHAR */
+    int shortnames;		/**< Always use short column names */
     int longnames;		/**< Don't shorten column names */
     int nocreat;		/**< Don't auto create database file */
     int curtype;		/**< Default cursor type */
@@ -140,6 +160,7 @@ typedef struct {
     int scale;			/**< Scale of column */
     int prec;			/**< Precision of column */
     int autoinc;		/**< AUTO_INCREMENT column */
+    int notnull;		/**< NOT NULL constraint on column */
     char *typename;		/**< Column type name or NULL */
     char *label;		/**< Column label or NULL */
 } COL;
@@ -168,14 +189,23 @@ typedef struct {
 typedef struct {
     int type, stype;	/**< ODBC and SQL types */
     int coldef, scale;	/**< from SQLBindParameter() */
-    int max, *lenp;	/**< Max. size, actual size of parameter buffer */
+    SQLLEN max;		/**< Max. size size of parameter buffer */
+    SQLLEN *lenp;	/**< Actual size of parameter buffer */
+    SQLLEN *lenp0;	/**< Actual size of parameter buffer, initial value */
     void *param;	/**< Parameter buffer */
     void *param0;	/**< Parameter buffer, initial value */
     int inc;		/**< Increment for paramset size > 1 */
     int need;		/**< True when SQL_LEN_DATA_AT_EXEC */
+    int bound;		/**< True when SQLBindParameter() called */ 
     int offs, len;	/**< Offset/length for SQLParamData()/SQLPutData() */
     void *parbuf;	/**< Buffer for SQL_LEN_DATA_AT_EXEC etc. */
     char strbuf[64];	/**< String buffer for scalar data */
+    int s3type;		/**< SQLite3 type */
+    int s3size;		/**< SQLite3 size */
+    void *s3val;	/**< SQLite3 value buffer */
+    int s3ival;		/**< SQLite3 integer value */
+    sqlite_int64 s3lival;	/**< SQLite3 64bit integer value */
+    double s3dval;	/**< SQLite3 float value */
 } BINDPARM;
 
 /**
@@ -209,6 +239,7 @@ typedef struct stmt {
     int naterr;			/**< Native error code */
     char sqlstate[6];		/**< SQL state for SQLError() */
     SQLCHAR logmsg[1024];	/**< Message for SQLError() */
+    int nowchar[2];		/**< Don't try to use WCHAR */
     int longnames;		/**< Don't shorten column names */
     int retr_data;		/**< SQL_ATTR_RETRIEVE_DATA */
     SQLUINTEGER rowset_size;	/**< Size of rowset */
