@@ -15,9 +15,9 @@
  * @file sqliteodbc.h
  * Header file for SQLite ODBC driver.
  *
- * $Id: sqliteodbc.h,v 1.42 2006/05/23 06:07:54 chw Exp chw $
+ * $Id: sqliteodbc.h,v 1.51 2009/01/08 11:57:14 chw Exp chw $
  *
- * Copyright (c) 2001-2006 Christian Werner <chw@ch-werner.de>
+ * Copyright (c) 2001-2009 Christian Werner <chw@ch-werner.de>
  *
  * See the file "license.terms" for information on usage
  * and redistribution of this file and for a
@@ -38,7 +38,7 @@
 #include "resourceos2.h"
 #endif
 
-#ifdef _WIN32
+#if defined(_WIN32) || defined(_WIN64)
 #include <windows.h>
 #if defined(HAVE_SQLITETRACE) && HAVE_SQLITETRACE
 #include <stdio.h>
@@ -47,13 +47,14 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <stdio.h>
-#include <stdlib.h>
 #include <unistd.h>
 #endif
-#if defined(HAVE_LOCALECONV) || defined(_WIN32)
+#include <stdlib.h>
+#if defined(HAVE_LOCALECONV) || defined(_WIN32) || defined(_WIN64)
 #include <locale.h>
 #endif
 #include <stdarg.h>
+#include <stddef.h>
 #include <string.h>
 #include <sql.h>
 #include <sqlext.h>
@@ -63,7 +64,7 @@
 #ifdef HAVE_IODBC
 #include <iodbcinst.h>
 #endif
-#if defined(HAVE_UNIXODBC) || defined(_WIN32)
+#if defined(HAVE_UNIXODBC) || defined(_WIN32) || defined(_WIN64)
 #include <odbcinst.h>
 #endif
 
@@ -85,6 +86,18 @@
 #define SQLROWCOUNT SQLUINTEGER
 #endif
 
+#ifndef HAVE_SQLSETPOSIROW
+#define SQLSETPOSIROW SQLUSMALLINT
+#endif
+
+#ifndef HAVE_SQLROWOFFSET
+#define SQLROWOFFSET SQLLEN
+#endif
+
+#ifndef HAVE_SQLROWSETSIZE
+#define SQLROWSETSIZE SQLULEN
+#endif
+
 struct dbc;
 struct stmt;
 
@@ -97,6 +110,10 @@ struct stmt;
 typedef struct {
     int magic;			/**< Magic cookie */
     int ov3;			/**< True for SQL_OV_ODBC3 */
+#if defined(_WIN32) || defined(_WIN64)
+    CRITICAL_SECTION cs;	/**< For serializing most APIs */
+    DWORD owner;		/**< Current owner of CS or 0 */
+#endif
     struct dbc *dbcs;		/**< Pointer to first DBC */
 } ENV;
 
@@ -116,6 +133,7 @@ typedef struct dbc {
     char *dsn;			/**< ODBC data source name */
     int timeout;		/**< Lock timeout value */
     long t0;			/**< Start time for SQLITE busy handler */
+    int busyint;		/**< Interrupt busy handler from SQLCancel() */
     int *ov3;			/**< True for SQL_OV_ODBC3 */
     int ov3val;			/**< True for SQL_OV_ODBC3 */
     int autocommit;		/**< Auto commit state */
@@ -157,6 +175,7 @@ typedef struct {
     int scale;			/**< Scale of column */
     int prec;			/**< Precision of column */
     int autoinc;		/**< AUTO_INCREMENT column */
+    int notnull;		/**< NOT NULL constraint on column */
     char *typename;		/**< Column type name or NULL */
     char *label;		/**< Column label or NULL */
 } COL;
@@ -185,11 +204,14 @@ typedef struct {
 typedef struct {
     int type, stype;	/**< ODBC and SQL types */
     int coldef, scale;	/**< from SQLBindParameter() */
-    int max, *lenp;	/**< Max. size, actual size of parameter buffer */
+    SQLLEN max;		/**< Max. size size of parameter buffer */
+    SQLLEN *lenp;	/**< Actual size of parameter buffer */
+    SQLLEN *lenp0;	/**< Actual size of parameter buffer, initial value */
     void *param;	/**< Parameter buffer */
     void *param0;	/**< Parameter buffer, initial value */
     int inc;		/**< Increment for paramset size > 1 */
     int need;		/**< True when SQL_LEN_DATA_AT_EXEC */
+    int bound;		/**< True when SQLBindParameter() called */ 
     int offs, len;	/**< Offset/length for SQLParamData()/SQLPutData() */
     void *parbuf;	/**< Buffer for SQL_LEN_DATA_AT_EXEC etc. */
     char strbuf[64];	/**< String buffer for scalar data */
